const fs = require('fs');
const path = require('path');

const Kynfig = function () {
  this.defaultConfig = fs.readFileSync(path.resolve('kynfig/default.json'), 'utf-8');
  this.folder = fs.readdirSync(path.resolve('kynfig'));
}

Kynfig.prototype.get = function (path) {
  const legs = path.split('.');
  let config = JSON.parse(this.defaultConfig);
  let result = {};

  if (typeof process.env.NODE_ENV !== 'undefined') {
    const nodeEnvFile = this.folder.some(file => file === `${process.env.NODE_ENV}.json`);

    if (!nodeEnvFile) throw new Error(`Error: NODE_ENV=${process.env.NODE_ENV} config file not found!`)

    let nodeEnvConfig = fs.readFileSync(`kynfig/${process.env.NODE_ENV}.json`, 'utf-8');
    config = transverse(config, JSON.parse(nodeEnvConfig), path);
  }

  legs.forEach(function (leg) {
    if (!Object.keys(result).length && config[leg]) {
      return result = config[leg];
    } else if (result[leg] || result[leg] === '') {
      return result = result[leg];
    }
    throw new Error(`Error: No property found ${leg} on path: ${path}`);
  });

  if (typeof result !== 'string') {
    result = JSON.stringify(result);
  }

  return result;
}

function transverse(target, object, path) {
  if (target === undefined) throw new Error(`Error: NODE_ENV=${process.env.NODE_ENV}. ${JSON.stringify(path)} not found on default file`)

  Object.keys(object).forEach(function (key) {
    if (typeof object[key] === 'object') {
      return transverse(target[key], object[key], path);
    }

    if (!object[key].toString().length) throw new Error(`Error: NODE_ENV=${process.env.NODE_ENV}. Property ${key} not found on default file`)

    target[key] = object[key];
  });

  return target;
}

module.exports = new Kynfig();
